

class EventType:
    """
    A class containing constant event types for use with event handling.

    These event types represent various events that can be received or handled
    from the Discord WebSocket API. They are used to identify the type of event
    that is triggered, such as user updates, messages, interactions, and more.
    """
    ANY = "ANY"
    READY="READY"
    PRESENCE_UPDATE="PRESENCE_UPDATE"
    MESSAGE_CREATE="MESSAGE_CREATE"
    MESSAGE_UPDATE="MESSAGE_UPDATE"
    MESSAGE_DELETE="MESSAGE_DELETE"
    MESSAGE_ACK="MESSAGE_ACK"
    INTERACTION_CREATE="INTERACTION_CREATE"
    USER_SETTINGS_PROTO_UPDATE="USER_SETTINGS_PROTO_UPDATE"
    SESSIONS_REPLACE="SESSIONS_REPLACE"
    PRESENCE_UPDATE="PRESENCE_UPDATE"
    GUILD_AUDIT_LOG_ENTRY_CREATE="GUILD_AUDIT_LOG_ENTRY_CREATE"
    FRIEND_REQUEST="RELATIONSHIP_ADD"
    DELETE_FRIEND="RELATIONSHIP_REMOVE"
    USER_NON_CHANNEL_ACK="USER_NON_CHANNEL_ACK"
    CALL_CREATE="CALL_CREATE"
    CALL_UPDATE="CALL_UPDATE"
    VOICE_STATE_UPDATE="VOICE_STATE_UPDATE"
    CALL_UPDATE="CALL_UPDATE"
    NOTIFICATION_CENTER_ITEMS_ACK="NOTIFICATION_CENTER_ITEMS_ACK"
    TYPING_START="TYPING_START"
    CONTENT_INVENTORY_INBOX_STALE="CONTENT_INVENTORY_INBOX_STALE"
    CHANNEL_UPDATE="CHANNEL_UPDATE"